@@ -34,7 +34,7 @@ def clamp_aa(src: vs.VideoNode, weak: vs.VideoNode, strong: vs.VideoNode, streng
     Clamp stronger AAs to weaker AAs.
 
     .. warning::
-        | This function has been deprecated! It will be removed in a future commit.
+        | This function has been deprecated in favor of vsaa.clamp_aa!
 
     Useful for clamping :py:func:`lvsfunc.aa.upscaled_sraa` or :py:func:`lvsfunc.aa.eedi3`
     to :py:func:`lvsfunc.aa.nnedi3` for a strong but more precise AA.
@@ -48,7 +48,6 @@ def clamp_aa(src: vs.VideoNode, weak: vs.VideoNode, strong: vs.VideoNode, streng
 
     :return:            Clip with clamped anti-aliasing.
     """
-
     warnings.warn('lvsfunc.clamp_aa: deprecated in favor of vsaa.clamp_aa!', DeprecationWarning)
 
     return vsaa.clamp_aa(src, weak, strong, strength)
@@ -59,7 +58,7 @@ def taa(clip: vs.VideoNode, aafun: Callable[[vs.VideoNode], vs.VideoNode] | Sing
     Perform transpose AA.
 
     .. warning::
-        | This function has been deprecated! It will be removed in a future commit.
+        | This function has been deprecated in favor of vsaa.transpose_aa!
 
     Example for nnedi3cl: taa(clip, nnedi3(opencl=True))
 
@@ -68,7 +67,6 @@ def taa(clip: vs.VideoNode, aafun: Callable[[vs.VideoNode], vs.VideoNode] | Sing
 
     :return:            Antialiased clip.
     """
-
     warnings.warn('lvsfunc.taa: deprecated in favor of vsaa.transpose_aa!', DeprecationWarning)
 
     if isinstance(aafun, SingleRater):
@@ -88,7 +86,7 @@ def nnedi3(opencl: bool = False, **override: Any) -> Callable[[vs.VideoNode], vs
     Generate nnedi3 antialiaser.
 
     .. warning::
-        | This function has been deprecated! It will be removed in a future commit.
+        | This function has been deprecated in favor of vsaa.Nnedi3!
 
     Dependencies:
 
@@ -113,7 +111,7 @@ def eedi3(opencl: bool = False, **override: Any) -> Callable[[vs.VideoNode], vs.
     Generate eedi3 antialiaser.
 
     .. warning::
-        | This function has been deprecated! It will be removed in a future commit.
+        | This function has been deprecated in favor of vsaa.Eedi3!
 
     Dependencies:
 
@@ -139,7 +137,7 @@ def nneedi3_clamp(clip: vs.VideoNode, strength: float = 1,
     Clamp eedi3 to nnedi3 for the purpose of reducing eedi3 artifacts.
 
     .. warning::
-        | This function has been deprecated! It will be removed in a future commit.
+        | This function has been deprecated in favor of vsaa.masked_clamp_aa!
 
     This should fix `every issue created by eedi3 <https://i.imgur.com/hYVhetS.jpg>`_.
 
@@ -154,8 +152,7 @@ def nneedi3_clamp(clip: vs.VideoNode, strength: float = 1,
 
     :return:                    Antialiased clip.
     """
-
-    warnings.warn('lvsfunc.nneedi3_clamp: deprecated in favor of vsaa.fine_aa!', DeprecationWarning)
+    warnings.warn('lvsfunc.nneedi3_clamp: deprecated in favor of vsaa.masked_clamp_aa!', DeprecationWarning)
 
     return vsaa.masked_clamp_aa(clip, strength, mthr, mask, opencl=opencl)
 
@@ -165,7 +162,7 @@ def transpose_aa(clip: vs.VideoNode, eedi3: bool = False, rep: int = 13) -> vs.V
     Tranpose and aa a clip multiple times using nnedi3/eedi3.
 
     .. warning::
-        | This function has been deprecated! It will be removed in a future commit.
+        | This function has been deprecated in favor of vsaa.fine_aa!
 
     This results in overall stronger anti-aliasing.
     Useful for shows like Yuru Camp with bad line-art problems.
@@ -185,7 +182,6 @@ def transpose_aa(clip: vs.VideoNode, eedi3: bool = False, rep: int = 13) -> vs.V
 
     :return:          Antialiased clip.
     """
-
     warnings.warn('lvsfunc.transpose_aa: deprecated in favor of vsaa.fine_aa!', DeprecationWarning)
 
     return vsaa.fine_aa(clip, True, vsaa.Eedi3() if eedi3 else vsaa.Nnedi3(), rep)
@@ -231,7 +227,6 @@ def upscaled_sraa(clip: vs.VideoNode,
 
     :raises ValueError:     ``rfactor`` is not above 1.
     """
-
     warnings.warn('lvsfunc.upscaled_sraa: deprecated in favor of vsaa.upscaled_sraa!', DeprecationWarning)
 
     if isinstance(supersampler, SuperSampler):
