@@ -25,7 +25,8 @@ def autodb_dpir(clip: vs.VideoNode, edgevalue: int = 24,
                 thrs: Sequence[tuple[float, float, float]] = [(1.5, 2.0, 2.0), (3.0, 4.5, 4.5), (5.5, 7.0, 7.0)],
                 matrix: Matrix | int | None = None,
                 kernel: Kernel | str = Bicubic(b=0, c=1/2),
-                cuda: bool = True, write_props: bool = False,
+                cuda: bool | Literal['trt'] | None = None,
+                write_props: bool = False,
                 **vsdpir_args: Any) -> vs.VideoNode:
     r"""
     Rewrite of fvsfunc.AutoDeblock that uses vspdir instead of dfttest to deblock.
@@ -65,7 +66,10 @@ def autodb_dpir(clip: vs.VideoNode, edgevalue: int = 24,
     :param kernel:          py:class:`vskernels.Kernel` object used for conversions between YUV <-> RGB.
                             This can also be the string name of the kernel
                             (Default: py:class:`vskernels.Bicubic(0, 0.5)`).
-    :param cuda:            Use CUDA backend if True, else CPU backend.
+    :param cuda:            Used to select backend.
+                            Use CUDA if True, CUDA TensorRT if 'trt', else CPU OpenVINO if False.
+                            If ``None``, it will detect your system's capabilities
+                            and select the fastest backend.
     :param write_props:     whether to write verbose props.
     :param vsdpir_args:     Additional args to pass to :py:func:`lvsfunc.deblock.vsdpir`.
 
@@ -112,6 +116,10 @@ def autodb_dpir(clip: vs.VideoNode, edgevalue: int = 24,
     if isinstance(kernel, str):
         kernel = get_kernel(kernel)()
 
+    vsdpir_final_args = dict(mode='deblock', cuda=cuda)
+    vsdpir_final_args |= vsdpir_args
+    vsdpir_final_args.pop('strength', None)
+
     nthrs = [tuple(x / 255 for x in thr) for thr in thrs]
 
     is_rgb = clip.format.color_family is vs.RGB
@@ -136,7 +144,7 @@ def autodb_dpir(clip: vs.VideoNode, edgevalue: int = 24,
     diffprev = core.std.PlaneStats(rgb, rgb[0] + rgb, prop='YPrev')
 
     db_clips = [
-        dpir(rgb, strength=st, mode='deblock', cuda=cuda, **vsdpir_args)
+        dpir(rgb, strength=st, **vsdpir_final_args)
         .std.SetFrameProp('Adb_DeblockStrength', intval=int(st)) for st in strs
     ]
 
@@ -328,6 +336,8 @@ def dpir(
 
     no_dpir_zones = list[Range]()
 
+    zoned_strength_clip = strength_clip
+
     if zones:
         cache_strength_clips = dict[float, vs.VideoNode]()
 
@@ -360,7 +370,7 @@ def dpir(
 
         if len(dpir_zones) <= 2:
             for rrange, sclip in dpir_zones.items():
-                zoned_strength_clip = replace_ranges(strength_clip, sclip, rrange)
+                zoned_strength_clip = replace_ranges(zoned_strength_clip, sclip, rrange)
         else:
             dpir_ranges_zones = {
                 range(*(
@@ -383,8 +393,6 @@ def dpir(
                 return strength_clip
 
             zoned_strength_clip = strength_clip.std.FrameEval(_select_sclip)
-    else:
-        zoned_strength_clip = strength_clip
 
     if None in {cuda, fp16}:
         try:
