@@ -13,5 +13,8 @@ __all__ = [
 
 
 class FPSColumn(ProgressColumn):
+    """Progress rendering."""
+
     def render(self, task: Task) -> Text:
+        """Render bar."""
         return Text(f"{task.speed or 0:.02f} fps")
