@@ -1,12 +1,30 @@
 from __future__ import annotations
 
-from enum import IntEnum, auto
+from dataclasses import dataclass
+from enum import Enum, IntEnum, auto
 from typing import Any, NamedTuple, Protocol, Tuple
 
 import vapoursynth as vs
 
 __all__ = [
-    'Coordinate', 'Direction', 'Position', 'Range', 'RegressClips', 'SceneChangeMode', 'Size', '_VideoNode'
+    '_VideoNode',
+    'Coordinate',
+    'Dar',
+    'Direction',
+    'IndexExists',
+    'IndexFile',
+    'IndexingType',
+    'IndexType',
+    'MISSING',
+    'MissingT',
+    'Position',
+    'Range',
+    'Region',
+    'RegressClips',
+    'SceneChangeMode',
+    'Shapes',
+    'Size',
+    'VSIdxFunction',
 ]
 
 Range = int | None | Tuple[int | None, int | None]
@@ -91,3 +109,51 @@ class _VideoNode(vs.VideoNode):
     """Use for asserting a VideoFormat exists."""
 
     format: vs.VideoFormat
+
+
+class IndexingType(str, Enum):
+    """Indexing types."""
+
+    DGI = '.dgi'
+    LWI = '.lwi'
+
+
+@dataclass
+class IndexFile:
+    """Type of index file."""
+
+    type: IndexingType
+    exists: bool
+
+
+class IndexType(IntEnum):
+    """Type of index."""
+
+    IMAGE = auto()
+    NONE = auto()
+
+
+class MissingT:
+    """Missing."""
+
+    pass
+
+
+MISSING = MissingT()
+
+
+class Dar(str, Enum):
+    """StrEnum signifying an analog television aspect ratio."""
+
+    WIDESCREEN = "widescreen"
+    WIDE = WIDESCREEN
+    FULLSCREEN = "fullscreen"
+    FULL = FULLSCREEN
+    SQUARE = "square"
+
+
+class Region(str, Enum):
+    """StrEnum signifying an analog television region."""
+
+    NTSC = "NTSC"
+    PAL = "PAL"
